package main

import (
	"fmt"
	"net"
)

func main() {
	host := "0.0.0.0"
	port := "8888"
	addr := net.JoinHostPort(host, port)
	fmt.Println(addr)                    // 0.0.0.0:8888
	fmt.Println(net.SplitHostPort(addr)) // 0.0.0.0 8888 <nil>

	// 获取本地所有 IP 信息
	fmt.Println("Interfaces:")
	ifaces, err := net.Interfaces()
	for _, iface := range ifaces {
		fmt.Println(iface.Index, iface.Name, iface.MTU, iface.Flags, iface.HardwareAddr)
		fmt.Println(iface.Addrs())
		fmt.Println(iface.MulticastAddrs())
	}

	fmt.Println("InterfaceAddrs:")
	addrs, err := net.InterfaceAddrs()
	for _, addr := range addrs {
		fmt.Println(addr.String(), addr.Network())
	}

	host = "www.xiexianbin.cn"
	// 根据域名查找 IP
	var ips []net.IP
	ips, err = net.LookupIP(host)
	if err != nil {
		fmt.Println(err.Error())
	}
	fmt.Println(ips)

	fmt.Println(net.LookupHost(host))

	// 解析 txt 记录
	mxs, err := net.LookupMX("xiexianbin.cn")
	if err != nil {
		fmt.Println(err)
	}
	for _, mx := range mxs {
		//&net.MX{Host:"mxn.mxhichina.com.", Pref:0x5}
		//&net.MX{Host:"mxw.mxhichina.com.", Pref:0xa}
		fmt.Printf("%#v\n", mx)
	}
	txts, err := net.LookupTXT("_dnsauth.ca.xiexianbin.cn ")
	if err != nil {
		fmt.Println(err)
	}
	for _, txt := range txts {
		fmt.Printf("%#v\n", txt)
	}

	// cidr
	ip, ipnet, err := net.ParseCIDR("172.20.0.1/24")
	fmt.Println(ip, ipnet, err)                             // 172.20.0.1 172.20.0.0/24 <nil>
	fmt.Println(ipnet.Contains(net.ParseIP("172.20.0.10"))) // true
	fmt.Println(ipnet.Contains(net.ParseIP("172.20.1.1")))  // false
	fmt.Println(ipnet.Network())                            // ip+net

	for _, s := range []string{"172.20.0.1", "::1", "foo.ip"} {
		ip := net.ParseIP(s)
		fmt.Println(ip) // [172.20.0.1 ::1 <nil>]
		if ip != nil {
			fmt.Println(ip.IsLinkLocalMulticast())
			fmt.Println(ip.IsLoopback())
			fmt.Println(ip.IsGlobalUnicast())
			fmt.Println(ip.IsPrivate())
		}
	}

	// 根据 IP 查找域名
	names, err := net.LookupAddr(ips[0].String())
	if err != nil {
		fmt.Println(err.Error())
	}
	fmt.Println(names)
}
